@@ -6,8 +6,9 @@ __all__ = [
 
 # This should always be 0.0.0 in main. Only update this after tagging
 # before release.
-__version__ = '10.11.0'
+__version__ = '11.17.0'
 
+import base64
 import contextlib
 import json
 import logging
@@ -59,6 +60,11 @@ SELECT_ADMIN_HEADER = 'Dropbox-API-Select-Admin'
 
 SELECT_USER_HEADER = 'Dropbox-API-Select-User'
 
+USER_AUTH = 'user'
+TEAM_AUTH = 'team'
+APP_AUTH = 'app'
+NO_AUTH = 'noauth'
+
 class RouteResult(object):
     """The successful result of a call to a route."""
 
@@ -176,8 +182,10 @@ class _DropboxTransport(object):
             refresh will request all available scopes for application
         """
 
-        if not (oauth2_access_token or oauth2_refresh_token):
-            raise BadInputException('OAuth2 access token or refresh token must be set')
+        if not (oauth2_access_token or oauth2_refresh_token or (app_key and app_secret)):
+            raise BadInputException(
+                'OAuth2 access token or refresh token or app key/secret must be set'
+            )
 
         if headers is not None and not isinstance(headers, dict):
             raise BadInputException('Expected dict, got {}'.format(headers))
@@ -293,6 +301,7 @@ class _DropboxTransport(object):
         self.check_and_refresh_access_token()
 
         host = route.attrs['host'] or 'api'
+        auth_type = route.attrs['auth']
         route_name = namespace + '/' + route.name
         if route.version > 1:
             route_name += '_v{}'.format(route.version)
@@ -315,6 +324,7 @@ class _DropboxTransport(object):
                                                   route_name,
                                                   route_style,
                                                   serialized_arg,
+                                                  auth_type,
                                                   request_binary,
                                                   timeout=timeout)
         decoded_obj_result = json.loads(res.obj_result)
@@ -350,9 +360,10 @@ class _DropboxTransport(object):
         :return:
         """
         can_refresh = self._oauth2_refresh_token and self._app_key
-        needs_refresh = self._oauth2_access_token_expiration and \
-            (datetime.utcnow() + timedelta(seconds=TOKEN_EXPIRATION_BUFFER)) >= \
-            self._oauth2_access_token_expiration
+        needs_refresh = self._oauth2_refresh_token and \
+            (not self._oauth2_access_token_expiration or
+            (datetime.utcnow() + timedelta(seconds=TOKEN_EXPIRATION_BUFFER)) >=
+            self._oauth2_access_token_expiration)
         needs_token = not self._oauth2_access_token
         if (needs_refresh or needs_token) and can_refresh:
             self.refresh_access_token(scope=self._scope)
@@ -407,6 +418,7 @@ class _DropboxTransport(object):
                             route_name,
                             route_style,
                             request_arg,
+                            auth_type,
                             request_binary,
                             timeout=None):
         """
@@ -418,6 +430,7 @@ class _DropboxTransport(object):
         :param route_style: The style of the route.
         :param str request_arg: A JSON-serializable Python object representing
             the argument for the route.
+        :param auth_type str
         :param Optional[bytes] request_binary: Bytes representing the binary
             payload. Use None if there is no binary payload.
         :param Optional[float] timeout: Maximum duration in seconds
@@ -432,6 +445,7 @@ class _DropboxTransport(object):
                                                   route_name,
                                                   route_style,
                                                   serialized_arg,
+                                                  auth_type,
                                                   request_binary,
                                                   timeout=timeout)
         # This can throw a ValueError if the result is not deserializable,
@@ -447,6 +461,7 @@ class _DropboxTransport(object):
                                        route_name,
                                        route_style,
                                        request_json_arg,
+                                       auth_type,
                                        request_binary,
                                        timeout=None):
         """
@@ -465,6 +480,7 @@ class _DropboxTransport(object):
                                                 route_name,
                                                 route_style,
                                                 request_json_arg,
+                                                auth_type,
                                                 request_binary,
                                                 timeout=timeout)
             except AuthError as e:
@@ -507,6 +523,7 @@ class _DropboxTransport(object):
                             func_name,
                             route_style,
                             request_json_arg,
+                            auth_type,
                             request_binary,
                             timeout=None):
         """
@@ -529,10 +546,25 @@ class _DropboxTransport(object):
         url = self._get_route_url(fq_hostname, func_name)
 
         headers = {'User-Agent': self._user_agent}
-        if host != HOST_NOTIFY:
+        auth_types = auth_type.replace(' ', '').split(',')
+        if (USER_AUTH in auth_types or TEAM_AUTH in auth_types) and self._oauth2_access_token:
             headers['Authorization'] = 'Bearer %s' % self._oauth2_access_token
             if self._headers:
                 headers.update(self._headers)
+        elif APP_AUTH in auth_types:
+            if self._app_key is None or self._app_secret is None:
+                raise BadInputException(
+                    'Client id and client secret are required for routes with app auth')
+            auth_header = base64.b64encode(
+                "{}:{}".format(self._app_key, self._app_secret).encode("utf-8")
+            )
+            headers['Authorization'] = 'Basic {}'.format(auth_header.decode("utf-8"))
+            if self._headers:
+                headers.update(self._headers)
+        elif auth_type == NO_AUTH:
+            pass
+        else:
+            raise BadInputException('Unhandled auth type: {}'.format(auth_type))
 
         # The contents of the body of the HTTP request
         body = None
